<?php

// We don't want to allow direct access to this
defined( 'ABSPATH' ) OR die( 'No direct script access allowed' );

class KCP_CSPGEN_Headers {

    // process the headers
    public function kp_process_headers( ) : void {

        // hook into our headers action
        add_action( 'send_headers', function( ) { 

            // get our generated headers
            $_gen_headers = $this -> kp_populate_header_array( );

            // loop over the generated header and add it
            foreach( $_gen_headers as $_k => $_v ) {

                // add the header
                header( $_k . ': ' . $_v, false );
            }

        }, PHP_INT_MAX );

        // check if we're going to apply these headers to the admin side as well
        if( get_our_option( 'apply_to_admin' ) ) {

            // we are, so let's hook into an admin specific action where we can apply them
            add_action( 'admin_init', function( ) {

                // get our generated headers
                $_gen_headers = $this -> kp_populate_header_array( );

                // loop over the generated header and add it
                foreach( $_gen_headers as $_k => $_v ) {

                    // add the header
                    header( $_k . ': ' . $_v, false );
                }

            }, PHP_INT_MAX );

        }

    }

    // returnable headers for backend display for manual implementation only
    public function kp_process_headers_for_display( ) : array {
        
        // just return
        return $this -> kp_populate_header_array( );
    }

    // populate the header array, we're going to utilize this for the autogeneration
    private function kp_populate_header_array( ) : array {

        // setup the return array
        $_ret = array( );

        // see if we're configured to include the Strict Transport Security header
        if( get_our_option( 'include_sts' ) ) {

            // get our directives, and set defaults if they are not set
            $_age = ( get_our_option( 'include_sts_max_age' ) ) ? get_our_option( 'include_sts_max_age' ) : 31536000;
            $_include = ( get_our_option( 'include_sts_subdomains' ) ) ? 'includeSubdomains;' : null;
            $_preload = ( get_our_option( 'include_sts_preload' ) ) ? 'preload;' : null;

            // trim the last semi-colon if needed
            if( $_include && $_preload ) {

                $_extras = rtrim( $_include . ' ' . $_preload, ';' );
            } else {

                // if either include or preload, remove the last semi-colon
                if( $_include ) {

                    $_extras = rtrim( $_include, ';' );
                } else {

                    $_extras = rtrim( $_preload, ';' );
                }
            }

            // set the header with directives
            $_ret['Strict-Transport-Security'] = "max-age=$_age; $_extras";
        }

        // see if we're configured to include the from options header
        if( get_our_option( 'include_ofs' ) ) {

            // set the header with our configured directives
            $_ret['X-Frame-Options'] = ( get_our_option( 'include_ofs_type' ) ) ? get_our_option( 'include_ofs_type' ) : 'DENY';
        }

        // see if we're configured to include the cross site scripting fix
        if( get_our_option( 'include_xss' ) ) {

            $_ret['X-Xss-Protection'] = '1; mode=block';
        }

        // see if we're configured to include nosniff
        if( get_our_option( 'include_mimesniffing' ) ) {

            $_ret['X-Content-Type-Options'] = 'nosniff';
        }

        // see if we're configured to include referrer policy
        if( get_our_option( 'include_referrer_policy' ) ) {

            $_ret['Referrer-Policy'] = ( get_our_option( 'include_referrer_policy_setting' ) ) ? get_our_option( 'include_referrer_policy_setting' ) : 'strict-origin';
        }

        // see if we're configured to include forced downloads
        if( get_our_option( 'include_download_options' ) ) {

            $_ret['X-Download-Options'] = 'noopen';
        }

        // see if we're configured to include cross domain origins
        if( get_our_option( 'include_crossdomain' ) ) {

            $_ret['X-Permitted-Cross-Domain-Policies'] = 'none';
        }

        // check if we're actually generating a content security policy
        if( get_our_option( 'generate_csp' ) ) {

            // fire up our chunk holder
            $_chunk = '';

            // generate our full CSP, if we're configured to do so
            $_chunk .= $this -> kp_cspgen_csp_string( );

            // see if we're configured to upgrade all requests, if so.. append it to the chunk.  IIS doesn't allow duplicate keys
            if( get_our_option( 'include_upgrade_insecure' ) ) {

                // the iis header
                $_chunk .= ' upgrade-insecure-requests;';
            }

            // add the content security policy header
            $_ret['Content-Security-Policy'] = $_chunk;
            $_ret['X-Content-Security-Policy'] = $_chunk;

        } else {

            // see if we're configured to upgrade all requests, if so.. append it to the chunk.  IIS doesn't allow duplicate keys
            if( get_our_option( 'include_upgrade_insecure' ) ) {

                // add the content security policy header
                $_ret['Content-Security-Policy'] = 'upgrade-insecure-requests;';
                $_ret['X-Content-Security-Policy'] = 'upgrade-insecure-requests;';
            }
            
        }

        // return the generated array
        return $_ret;

    }

    // generate a full string for the content security policy
    private function kp_cspgen_csp_string( ) : ?string {

        // hold our generated CSP array
        $_generated = array( );

        // get our site's host
        $_our_host = parse_url( get_site_url( ), PHP_URL_HOST );

        // get our CPT data, if it exists
        $_qry = new WP_Query( array( 'post_type' => 'kcp_csp', 'posts_per_page' => 1, 'post_status' => 'draft' ) );
        $_rs = $_qry -> get_posts( );

        // if it exists: css, script, connect, frame, media, font, img, default
        if( $_rs ) {

            // populate the generated CSP
            $_generated = maybe_unserialize( $_rs[0] -> post_content );
        }

        // setup some variables to hold our individual setions
        $_style = '';
        $_script = '';
        $_font = '';
        $_img = '';
        $_connect = '';
        $_frame = '';
        $_frame_anc = '';
        $_media = '';
        $_form = '';
        $_default = '';

        // start the style block
        $_style .= ' style-src \'self\' \'unsafe-inline\' ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['css'] ) ) {

            // they exist, so include them
            $_style .= implode( ' ', $_generated['css'] ) . ' ';

        }

        // check if we should include the default wordpress stuff
        if( get_our_option( 'include_wordpress_defaults' ) ) {
            $_style .= '*.googleapis.com *.gstatic.com';
        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_styles' ) ) ) {
            $_style .= ' ' . get_our_option( 'generate_csp_custom_styles' );
        }

        // end the style block
        $_style .= ';';

        // start the script block
        $_script .= ' script-src \'self\' \'unsafe-inline\' \'unsafe-eval\' ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['script'] ) ) {

            // they exist, so include them
            $_script .= implode( ' ', $_generated['script'] ) . ' ';

        }

        // check if we should include the default wordpress stuff
        if( get_our_option( 'include_wordpress_defaults' ) ) {
            $_script .= '*.g.doubleclick.net *.google-analytics.com *.google.com *.googletagmanager.com *.gstatic.com';
        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_scripts' ) ) ) {
            $_script .= ' ' . get_our_option( 'generate_csp_custom_scripts' );
        }

        // end the script block
        $_script .= ';';

        // start the font chunk
        $_font .= ' font-src \'self\' data: ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['font'] ) ) {

            // they exist, so include them
            $_font .= implode( ' ', $_generated['font'] ) . ' ';

        }

        // check if we should include the default wordpress stuff
        if( get_our_option( 'include_wordpress_defaults' ) ) {
            $_font .= '*.gstatic.com *.bootstrapcdn.com';
        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_fonts' ) ) ) {
            $_font .= ' ' . get_our_option( 'generate_csp_custom_fonts' );
        }

        // end the font chunk
        $_font .= ';';

        // start the images block
        $_img .= ' img-src \'self\' \'unsafe-inline\' \'unsafe-eval\' data: ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['img'] ) ) {

            // they exist, so include them
            $_img .= implode( ' ', $_generated['img'] ) . ' ';

        }

        // check if we should include the default wordpress stuff
        if( get_our_option( 'include_wordpress_defaults' ) ) {
            $_img .= '*.googletagmanager.com *.w.org *.gravatar.com *.google.com *.google-analytics.com *.gstatic.com';
        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_images' ) ) ) {
            $_img .= ' ' . get_our_option( 'generate_csp_custom_images' );
        }

        // end the images chunk
        $_img .= ';';

        // start the ajaxian chunk
        $_connect .= ' connect-src \'self\' ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['connect'] ) ) {

            // they exist, so include them
            $_connect .= implode( ' ', $_generated['connect'] ) . ' ';

        }
        
        // check if we should include the default wordpress stuff
        if( get_our_option( 'include_wordpress_defaults' ) ) {
            $_connect .= '*.google-analytics.com *.wpengine.com yoast.com *.google.com *.g.doubleclick.net';
        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_connect' ) ) ) {
            $_connect .= ' ' . get_our_option( 'generate_csp_custom_connect' );
        }

        // end the ajaxian chunk
        $_connect .= ';';

        // start the frame chunk
        $_frame .= ' frame-src \'self\' ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['frame'] ) ) {

            // they exist, so include them
            $_frame .= implode( ' ', $_generated['frame'] ) . ' ';

        }        

        // check if we should include the default wordpress stuff
        if( get_our_option( 'include_wordpress_defaults' ) ) {
            $_frame .= '*.g.doubleclick.net *.google.com *.fls.doubleclick.net';
        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_frames' ) ) ) {
            $_frame .= ' ' . get_our_option( 'generate_csp_custom_frames' );
        }

        // end the frame chunk
        $_frame .= ';';

        // start the frame chunk
        $_frame_anc .= ' frame-ancestors \'self\' ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['frame'] ) ) {

            // they exist, so include them
            $_frame_anc .= implode( ' ', $_generated['frame'] ) . ' ';

        }

        // check if we should include the default wordpress stuff
        if( get_our_option( 'include_wordpress_defaults' ) ) {
            $_frame_anc .= '*.g.doubleclick.net *.google.com *.fls.doubleclick.net';
        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_frames' ) ) ) {
            $_frame_anc .= ' ' . get_our_option( 'generate_csp_custom_frames' );
        }

        // end the frame chunk
        $_frame_anc .= ';';

        // start the media chunk
        $_media .= ' media-src \'self\' ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['media'] ) ) {

            // they exist, so include them
            $_media .= implode( ' ', $_generated['media'] ) . ' ';

        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_media' ) ) ) {
            $_media .= ' ' . get_our_option( 'generate_csp_custom_media' );
        }

        // end the media chunk
        $_media .= ';';

        // start the form chunk
        $_form .= ' form-action \'self\' ';

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_forms' ) ) ) {
            $_form .= ' ' . get_our_option( 'generate_csp_custom_forms' );
        }

        // end the form chunk
        $_form .= ';';

        // start the default chunk
        $_default .= ' default-src \'self\' ';

        // include the generated stylesheets if applicable
        if( isset( $_generated['default'] ) ) {

            // they exist, so include them
            $_default .= implode( ' ', $_generated['default'] ) . ' ';

        }

        // let's see if we have a value for the extras, and append it
        if( ! empty( get_our_option( 'generate_csp_custom_defaults' ) ) ) {
            $_default .= ' ' . get_our_option( 'generate_csp_custom_defaults' );
        }

        // end the default chunk
        $_default .= ';';

        // return our generated string
        return $this -> remove_duplicates( $_style ) . 
        $this -> remove_duplicates( $_script ) . 
        $this -> remove_duplicates( $_font ) . 
        $this -> remove_duplicates( $_img ) . 
        $this -> remove_duplicates( $_connect ) . 
        $this -> remove_duplicates( $_frame ) . 
        $this -> remove_duplicates( $_frame_anc ) . 
        $this -> remove_duplicates( $_media ) . 
        $this -> remove_duplicates( $_form ) . 
        $this -> remove_duplicates( $_default );

    }

    // remove the duplicates from the string
    protected function remove_duplicates( string $_str ) : string {

        // remove the duplicate strings
        return implode( ' ', array_unique( explode( ' ', $_str ) ) );

    }

}
